#!/usr/bin/python
from cStringIO import StringIO
from watchedinstall import *
import unittest


__test__ = dict(
read_program_test = """
# Calling env -h makes it complain and exit with 1
>>> with read_program(['/usr/bin/env', '-h']): pass
Traceback (most recent call last):
    ...
CalledProcessError: Command '/usr/bin/env' returned non-zero exit status 1
""",
)


class ParseArgumentsTest(unittest.TestCase):
    def test_missing_argument(self):
        """Require a positional argument."""
        p = make_option_parser()
        try:
            parse_arguments(p, [])
        except OptionValueError, err:
            pass
        
        self.assertEqual(str(err), 'Missing required process ID or file name.')
    
    def test_builtin_exludes(self):
        """No error when --builtin-excludes on command line."""
        p = make_option_parser()
        self.assertTrue(parse_arguments(p, ['--builtin-excludes']))
        

class RadmindEncodeTest(unittest.TestCase):
    """Tests Radmind-style string encoding."""
    def test_encode_space(self):
        """Encode space in path as \b."""
        encoded = radmind_encode('/Library/Application Support')
        self.assertEqual(encoded, r'/Library/Application\bSupport')

    def test_encode_tab(self):
        """Encode tab as \t."""
        # Note \t in first use is a tab, and \t in second is literal '\t'
        encoded = radmind_encode('FieldOne\tFieldTwo')
        self.assertEqual(encoded, r'FieldOne\tFieldTwo')
        
    def test_encode_newline(self):
        """Encode newline as \n."""
        encoded = radmind_encode('LineOne\nLineTwo')
        self.assertEqual(encoded, r'LineOne\nLineTwo')
    
    def test_encode_carriagereturn(self):
        """Encode carriage return as \r."""
        encoded = radmind_encode('LineOne\rLineTwo')
        self.assertEqual(encoded, r'LineOne\rLineTwo')
    
    def test_encode_backslash(self):
        """Encode a backslash as \\."""
        encoded = radmind_encode(r'C:\windows')
        self.assertEqual(encoded, r'C:\\windows')


class CreateDiffTest(unittest.TestCase):
    """Tests creating diff format."""
    def test_simple_diff(self):
        """Write a diff to a file-like object."""
        out = StringIO()
        transcript = """
+/Library
-/Applications
 /usr
!/System
"""
        create_diff(transcript, out)
        self.assertEqual(out.getvalue(), transcript)
        
        
if __name__ == "__main__":
    import doctest
    import sys
    import watchedinstall
    
    
    verbose = '-v' in sys.argv
    # Test watchedinstall doctests
    doctest.testmod(watchedinstall, verbose=verbose)
    # Test this module's doctests
    doctest.testmod(verbose=verbose)
    # Test this module's unittests
    unittest.main()
